annotate_trace_memory

Function annotate_trace_memory 

pub fn annotate_trace_memory(addr: *const ())
Ask DRD to report every access to the specified address

Trace all load and store activity that touches at least the single byte at the address addr.