RawFd

Type Alias RawFd 

pub type RawFd = c_int;
The raw file descriptor number

This type has no relationship to the standard library type definition of RawFd besides they are wrapping the same type on unix systems.