macro_rules! format_cstring {
($($args:tt)*) => { ... };
}Available on crate feature
alloc only.Expand description
Convenience macro to create a \0-byte terminated alloc::ffi::CString from a format string
This macro requires the alloc feature. In allocation-free builds, use valgrind_print
instead.
The format string passed to this macro must not contain or end with a \0-byte.
Safety
The same safety conditions as to the cstring macro apply here